@@ -67,7 +67,9 @@ answer_by_chain_of_thought <- function(
   }
 
   extraction_fn <- function(llm_response) {
-    if (!extract_from_finish_brackets) return(llm_response)
+    if (!extract_from_finish_brackets) {
+      return(llm_response)
+    }
     extraction_fn_finish(llm_response, extraction_lenience)
   }
 
