@@ -21,8 +21,16 @@ skip_test_if_no_ollama <- function(do_not_run = FALSE) {
     )
   }
 
-  # Define the request URL for the local Ollama server
-  url <- "http://localhost:11434"
+  if (!ollama_available()) {
+    return(testthat::skip("skip test: Ollama not available"))
+  }
+
+  return(invisible(NULL))
+}
+
+ollama_available <- function(
+  url = Sys.getenv("OLLAMA_URL", unset = "http://localhost:11434")
+) {
   req <- httr2::request(url)
 
   # Try performing the request and catch any errors
@@ -40,42 +48,47 @@ skip_test_if_no_ollama <- function(do_not_run = FALSE) {
     }
   )
 
-  if (!ollama_running) {
-    return(testthat::skip("skip test: Ollama not available"))
-  }
-
-  return(invisible(NULL))
+  return(ollama_running)
 }
 
 #' Skip tests if OpenAI model is unavailable
 #'
-#' Helper function for unit tests that skips the test if the OpenAI model is not available.
-#' The function sends a request to the OpenAI API to check if the model is available.
+#' Helper function for unit tests that skips the test if the OpenAI API is not available.
+#' The function sends a request to the OpenAI API to check available models.
 #' If there is no valid API key or the request fails, the test is skipped with an appropriate message.
 #'
 #' @param do_not_run Logical indicating whether to skip the test even if the
-#' OpenAI model is available. Temporarily set to `TRUE` to skip the tests
-#' @param model The OpenAI model to check availability for. Default is "gpt-4o-mini"
+#' OpenAI API is available. Temporarily set to `TRUE` to skip the tests
 #'
 #' @return testthat skip object or NULL
 #'
 #' @noRd
 #' @keywords internal
-skip_test_if_no_openai <- function(do_not_run = FALSE, model = "gpt-4o-mini") {
+skip_test_if_no_openai <- function(do_not_run = FALSE) {
   if (do_not_run) {
     return(
       testthat::skip("skip test: Skipping test even if OpenAI is available")
     )
   }
 
-  url <- paste0("https://api.openai.com/v1/models/", model)
-  api_key <- Sys.getenv("OPENAI_API_KEY")
+  if (!openai_available()) {
+    return(testthat::skip("skip test: OpenAI not available"))
+  }
+
+  return(invisible(NULL))
+}
+
+openai_available <- function(
+  url = Sys.getenv("OPENAI_URL", unset = "https://api.openai.com"),
+  api_key = Sys.getenv("OPENAI_API_KEY")
+) {
+  models_endpoint <- paste0(url, "/v1/models")
 
   if (is.null(api_key) | api_key == "") {
     return(testthat::skip("skip test: OpenAI API key not found"))
   }
 
-  req <- httr2::request(url) |>
+  req <- httr2::request(models_endpoint) |>
     httr2::req_headers(
       "Content-Type" = "application/json",
       "Authorization" = paste("Bearer", api_key)
@@ -95,9 +108,5 @@ skip_test_if_no_openai <- function(do_not_run = FALSE, model = "gpt-4o-mini") {
     }
   )
 
-  if (!openai_available) {
-    return(testthat::skip("skip test: OpenAI model not available"))
-  }
-
-  return(invisible(NULL))
+  return(openai_available)
 }
