@@ -64,6 +64,17 @@ NULL
     #'  See `$add_handler_fn()`
     handler_fns = list(),
 
+    #' @field stream_callback
+    #' Optional callback function for streaming tokens/chunks. If set, this function will be
+    #' called with arguments `(chunk, meta)` where `chunk` is the latest
+    #' streamed text chunk and `meta` is a list with fields including `llm_provider`,
+    #' `chat_history`, and `partial_response` to provide more context about the current
+    #' prompt that is being replied to. You may use this to implement custom streaming behaviour;
+    #' see `vignette(`"streaming_shiny_ipc", "tidyprompt")` for an example of how this
+    #' function is used to stream a non-blocking async LLM response to a Shiny app using the
+    #' 'ipc' package.
+    stream_callback = NULL,
+
     #' @field pre_prompt_wraps
     #' A list of prompt wraps that will be applied to any prompt evaluated
     #' by this [llm_provider-class] object, before any prompt-specific
@@ -134,8 +145,9 @@ NULL
       api_key = NULL,
       api_type = "unspecified"
     ) {
-      if (length(parameters) > 0 && is.null(names(parameters)))
+      if (length(parameters) > 0 && is.null(names(parameters))) {
         stop("parameters must be a named list")
+      }
 
       private$complete_chat_function <- complete_chat_function
       self$parameters <- parameters
@@ -157,7 +169,9 @@ NULL
     #'
     #' @return The modified [llm_provider-class] object
     set_parameters = function(new_parameters) {
-      if (length(new_parameters) == 0) return(self)
+      if (length(new_parameters) == 0) {
+        return(self)
+      }
 
       stopifnot(
         is.list(new_parameters),
@@ -213,7 +227,7 @@ NULL
         message(chat_history$content[nrow(chat_history)])
       }
 
-      if (self$verbose)
+      if (self$verbose) {
         message(
           crayon::bold(
             glue::glue(
@@ -221,6 +235,7 @@ NULL
             )
           )
         )
+      }
 
       environment(private$complete_chat_function) <- environment()
       response <- private$complete_chat_function(chat_history)
@@ -241,7 +256,7 @@ NULL
       http$requests[[1]] <- response$http$request
       http$responses[[1]] <- response$http$response
 
-      while (TRUE) {
+      repeat {
         for (handler_fn in self$handler_fns) {
           response <- handler_fn(response, self)
           http$requests[[length(http$requests) + 1]] <- response$http$request
@@ -284,8 +299,8 @@ NULL
         ) {
           chat_history_new_print <- chat_history_new |>
             dplyr::filter(
-              (is.na(tool_result) | tool_result == FALSE),
-              (is.na(tool_call) | tool_call == FALSE)
+              (is.na(tool_result) | !tool_result),
+              (is.na(tool_call) | !tool_call)
             )
         } else {
           chat_history_new_print <- chat_history_new
@@ -296,14 +311,17 @@ NULL
         }
       }
 
-      if (isTRUE(response$`break`))
+      if (isTRUE(response$`break`)) {
         warning(
           paste0(
             "Chat completion was interrupted by a handler break"
           )
         )
+      }
 
-      if (self$verbose) return(invisible(response))
+      if (self$verbose) {
+        return(invisible(response))
+      }
 
       return(response)
     },
@@ -368,7 +386,9 @@ NULL
         "name"
       )
       missing <- setdiff(needed, names(prompt_wrap))
-      for (nm in missing) prompt_wrap[[nm]] <- NULL
+      for (nm in missing) {
+        prompt_wrap[[nm]] <- NULL
+      }
 
       class(prompt_wrap) <- unique(c(
         "provider_prompt_wrap",
